@@ -1,7 +1,11 @@
-from docker import Client
-from docker import tls
-import ssl
 import os
+import ssl
+
+from docker import tls
+
+from ebcli.bundled._compose.docker_py_handler import client
+
+from docker import APIClient
 
 
 def docker_client():
@@ -32,4 +36,4 @@ def docker_client():
         )
 
     timeout = int(os.environ.get('DOCKER_CLIENT_TIMEOUT', 60))
-    return Client(base_url=base_url, tls=tls_config, version='1.17', timeout=timeout)
+    return APIClient(base_url=base_url, tls=tls_config, version=client.api_version().base_version, timeout=timeout)
