@@ -1,4 +1,4 @@
-# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -10,22 +10,26 @@
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
-
-import re
 import argparse
 import os
 
 from ..core import io, fileoperations, hooks
 from ..core.abstractcontroller import AbstractBaseController
-from ebcli.operations.commonops import is_platform_arn
 from ..lib import elasticbeanstalk, utils
 from ..objects.exceptions import NotFoundError, AlreadyExistsError, \
     InvalidOptionsError
+from ebcli.objects.platform import PlatformVersion
 from ..objects.requests import CreateEnvironmentRequest
 from ..objects.tier import Tier
-from ..operations import saved_configs, commonops, createops, composeops
+from ..operations import(
+    commonops,
+    composeops,
+    createops,
+    saved_configs,
+    solution_stack_ops
+)
 from ..resources.strings import strings, prompts, flag_text
-
+from ..resources.statics import elb_names
 
 class CreateController(AbstractBaseController):
     class Meta:
@@ -129,6 +133,7 @@ class CreateController(AbstractBaseController):
         elb_type = self.app.pargs.elb_type
         source = self.app.pargs.source
         process = self.app.pargs.process
+        region = self.app.pargs.region
         interactive = False if env_name else True
         platform_arn = None
         solution = None
@@ -141,22 +146,25 @@ class CreateController(AbstractBaseController):
         if single and scale:
             raise InvalidOptionsError(strings['create.singleandsize'])
 
+        if single and elb_type:
+            raise InvalidOptionsError(strings['create.single_and_elb_type'])
+
         app_name = self.get_app_name()
 
         # get tags
-        tags = get_and_validate_tags(tags)
+        tags = createops.get_and_validate_tags(tags)
 
         #load solution stack
         if not solution_string:
-            solution_string = commonops.get_default_solution_stack()
+            solution_string = solution_stack_ops.get_default_solution_stack()
 
         # Test out sstack and tier before we ask any questions (Fast Fail)
         if solution_string:
-            if is_platform_arn(solution_string):
+            if PlatformVersion.is_valid_arn(solution_string):
                 platform_arn = solution_string
             else:
                 try:
-                    solution = commonops.get_solution_stack(solution_string)
+                    solution = solution_stack_ops.find_solution_stack_from_string(solution_string)
                 except NotFoundError:
                     raise NotFoundError('Platform ' + solution_string +
                                         ' does not appear to be valid')
@@ -177,13 +185,13 @@ class CreateController(AbstractBaseController):
 
         # If we still dont have what we need, ask for it
         if not solution_string:
-            solution = commonops.prompt_for_solution_stack()
+            solution = solution_stack_ops.get_solution_stack_from_customer()
 
-        if solution is not None:
-            if is_platform_arn(solution.version):
-                platform_arn = solution.version
+        if solution:
+            if isinstance(solution, PlatformVersion) and PlatformVersion.is_valid_arn(solution.arn):
+                platform_arn = solution
                 solution = None
-            elif solution.platform == 'Multi-container Docker' and iprofile is None:
+            elif solution.language_name == 'Multi-container Docker' and not iprofile:
                 io.log_warning(prompts['ecs.permissions'])
 
         if not env_name:
@@ -225,7 +233,7 @@ class CreateController(AbstractBaseController):
             key_name = commonops.get_default_keyname()
 
         if not elb_type and interactive and not single:
-            elb_type = get_elb_type()
+            elb_type = get_elb_type(region)
 
         database = self.form_database_object()
         vpc = self.form_vpc_object()
@@ -256,6 +264,11 @@ class CreateController(AbstractBaseController):
         env_request.option_settings += envvars
 
         process_app_version = fileoperations.env_yaml_exists() or process
+
+        # avoid prematurely timing out in the CLI when an environment is launched with a RDS DB
+        if not timeout and database:
+            timeout = 15
+
         createops.make_new_env(env_request,
                                branch_default=branch_default,
                                process_app_version=process_app_version,
@@ -424,37 +437,26 @@ def get_cname(env_name):
             break
     return cname
 
-def get_elb_type():
+
+def get_elb_type(region):
     io.echo()
     io.echo('Select a load balancer type')
-    result = utils.prompt_for_item_in_list(["classic", "application"], default=1)
+    result = utils.prompt_for_item_in_list(elb_types(region), default=1)
     elb_type = result
 
     return elb_type
 
 
+def elb_types(region):
+    types = [elb_names.CLASSIC_VERSION, elb_names.APPLICATION_VERSION]
 
-def get_and_validate_tags(tags):
-    if not tags:
-        return []
+    if not region:
+        region = commonops.get_default_region()
 
-    tags = tags.strip().strip('"').strip('\'')
-    tags = tags.split(',')
-    tag_list = []
-    if len(tags) > 7:
-        raise InvalidOptionsError(strings['tags.max'])
-    for t in tags:
-        # validate
-        if not re.match('^[\w\s.:/+%@-]{1,128}=[\w\s.:/+=@-]{0,256}$', t):
-            raise InvalidOptionsError(strings['tags.invalidformat'])
-        else:
-            # build tag
-            key, value = t.split('=', 1)
-            tag_list.append(
-                {'Key': key,
-                 'Value': value}
-            )
-    return tag_list
+    if region not in ['cn-north-1', 'us-gov-west-1']:
+        types.append(elb_names.NETWORK_VERSION)
+
+    return types
 
 
 def get_and_validate_envars(envvars):
