@@ -1,4 +1,4 @@
-# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -20,7 +20,7 @@ from ..core.abstractcontroller import AbstractBaseController
 from ..lib import utils
 from ..objects.exceptions import NoEnvironmentForBranchError, \
     InvalidOptionsError
-from ..operations import commonops, deployops, composeops
+from ..operations import commonops, deployops, composeops, solution_stack_ops
 from ..resources.strings import strings, flag_text
 
 LOG = minimal_logger(__name__)
