@@ -1,4 +1,4 @@
-# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -20,6 +20,8 @@ import stat
 import sys
 import zipfile
 import yaml
+import warnings
+from pathspec import PathSpec
 
 from cement.utils.misc import minimal_logger
 from ebcli.objects.buildconfiguration import BuildConfiguration
@@ -188,31 +190,6 @@ def save_to_aws_config(access_key, secret_key):
     set_user_only_permissions(aws_config_location)
 
 
-def read_credentials_from_aws_dir():
-    config = configparser.ConfigParser()
-    if not os.path.isdir(aws_config_folder):
-        os.makedirs(aws_config_folder)
-
-    # The credential file credentials take precedence over config credentials so we check those first.
-    from ..lib import aws
-    profile = aws.get_profile()
-    config.read(aws_credentials_location)
-
-    if profile not in config.sections():
-        LOG.debug("Credentials not found in aws credentials file; trying config file")
-        profile = ebcli_section
-        config.read(aws_config_location)
-
-        if profile not in config.sections():
-            LOG.debug("Credentials not found in aws config file")
-            return None, None
-
-    access_key = _get_option(config, profile, aws_access_key, None)
-    secret_key = _get_option(config, profile, aws_secret_key, None)
-
-    return access_key, secret_key
-
-
 def set_user_only_permissions(location):
     """
     Sets permissions so that only a user can read/write (chmod 400).
@@ -250,6 +227,8 @@ def set_all_unrestricted_permissions(location):
     """
     os.chmod(location, stat.S_IRWXU | stat.S_IRWXG | stat.S_IRWXO)
 
+def remove_execute_access_from_group_and_other_users(location):
+    os.chmod(location, stat.S_IRWXU | stat.S_IRGRP | stat.S_IROTH)
 
 def get_current_directory_name():
     dirname, filename = os.path.split(os.getcwd())
@@ -439,6 +418,19 @@ def delete_app_versions():
         os.chdir(cwd)
 
 
+def zip_append_archive(target_file, source_file):
+    zip_source = zipfile.ZipFile(source_file, 'r')
+    zip_target = zipfile.ZipFile(target_file, 'a')
+    with warnings.catch_warnings():
+        # Ignore UserWarning raised by zip module for zipping modules.
+        warnings.simplefilter('ignore', category=UserWarning)
+        for filename in zip_source.namelist():
+            zf = zip_source.read(filename)
+            zip_target.writestr(filename, zf)
+    zip_target.close()
+    zip_source.close()
+
+
 def zip_up_folder(directory, location, ignore_list=None):
     cwd = os.getcwd()
     try:
@@ -853,60 +845,17 @@ def directory_empty(location):
 
 
 def get_ebignore_list():
-    EB_IGNORE_FILE_NAME = '.ebignore'
-    location = get_project_file_full_location(EB_IGNORE_FILE_NAME)
+    location = get_ebignore_location()
 
     if not os.path.isfile(location):
         return None
 
-    '''
-    This library will parse the ignore file, compare it to the current files
-    and give us a list of files to ignore
-    '''
-    # Patch iter_tree to not throw recursion error on non recursive links
-    from pathspec import pathspec, util
-    def iter_tree(root):
-        """
-        Walks the specified root path for all files.
-        *root* (``str``) is the root directory to search for files.
-        Raises ``RecursionError`` if recursion is detected.
-        Returns an ``Iterable`` yielding each file path (``str``) relative to
-        *root*.
-        .. _`recursion`: http://docs.python.org/2/library/os.html#os.walk
-        """
-        # Keep track of files encountered. Map real path to relative path.
-        memo = {}
-
-        root = os.path.abspath(root)
-        for parent, _dirs, files in os.walk(root, followlinks=True):
-            # Get parent path relative to root path.
-            parent = os.path.relpath(parent, root)
-
-            # Check for recursion.
-            real = os.path.realpath(parent)
-            if real in memo:
-                abspath = os.path.abspath(parent)
-                if real != abspath and real in abspath:
-                    # if real is a parent of current parent
-                    raise util.RecursionError(real_path=real, first_path=memo[real], second_path=parent)
-                else:
-                    # not recursion, just a sideways link
-                    continue
-
-            memo[real] = parent
-
-            # Yield files.
-            for path in files:
-                if parent != '.':
-                    path = os.path.join(parent, path)
-                yield path
-    util.iter_tree = iter_tree
-
-    with open(location, 'r') as f:
-        spec = pathspec.PathSpec.from_lines('gitignore', f)
+    with codecs.open(location, 'r', encoding='utf-8') as f:
+        spec = PathSpec.from_lines('gitwildmatch', f)
 
     ignore_list = [f for f in spec.match_tree(get_project_root())]
-    ignore_list.append(EB_IGNORE_FILE_NAME)
+    ignore_list.append('.ebignore')
+
     return ignore_list
 
 
@@ -981,6 +930,10 @@ def get_project_file_full_location(location):
         os.chdir(cwd)
 
 
+def get_ebignore_location():
+    return get_project_file_full_location('.ebignore')
+
+
 def get_eb_file_full_location(location):
     return get_project_file_full_location(beanstalk_directory + location)
 
