@@ -1,4 +1,4 @@
-# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -46,31 +46,17 @@ def delete_platform(arn):
                           PlatformArn=arn,
                           DeleteResources=True)
 
-def _make_equal_filter(filter_type, filter_value):
-    return { 'Type': filter_type, 'Operator': '=', 'Values': [filter_value] }    
 
-def list_platform_versions(platform_name=None, platform_version=None, status=None, owner=None):
+def list_platform_versions(filters=None):
     kwargs = dict()
-    result_filters = []
 
-    if platform_name is not None:
-        result_filters.append(_make_equal_filter('PlatformName', platform_name))
-
-    if platform_version is not None:
-        result_filters.append(_make_equal_filter('PlatformVersion', platform_version))
-
-    if status:
-        result_filters.append(_make_equal_filter('PlatformStatus', status))
-
-    if owner:
-        result_filters.append(_make_equal_filter('PlatformOwner', owner))
-
-    kwargs['Filters'] = result_filters
+    if filters:
+        kwargs['Filters'] = filters
 
     LOG.debug('Inside list_platform_versions api wrapper')
     platforms, nextToken = _list_platform_versions(kwargs)
 
-    while nextToken is not None:
+    while nextToken:
         time.sleep(0.1)  # To avoid throttling we sleep for 100ms before requesting the next page
         next_platforms, nextToken = _list_platform_versions(kwargs, nextToken)
         platforms = platforms + next_platforms
@@ -81,7 +67,7 @@ def list_platform_versions(platform_name=None, platform_version=None, status=Non
 def describe_platform_version(arn):
     LOG.debug('Inside describe_platform_version api wrapper')
     return _make_api_call('describe_platform_version',
-                          PlatformArn=arn)
+                          PlatformArn=arn)['PlatformDescription']
 
 
 def _list_platform_versions(kwargs, nextToken=None):
@@ -299,7 +285,8 @@ def _api_to_environment(api_dict, want_solution_stack = False):
         cname=api_dict.get('CNAME', 'UNKNOWN'),
         option_settings=api_dict.get('OptionSettings'),
         is_abortable=api_dict.get('AbortableOperationInProgress', False),
-        environment_links=api_dict.get('EnvironmentLinks')
+        environment_links=api_dict.get('EnvironmentLinks'),
+        environment_arn=api_dict.get('EnvironmentArn')
     )
     return env
 
@@ -425,7 +412,10 @@ def get_available_solution_stacks(fail_on_empty_response=True):
 
     solution_stacks = [SolutionStack(s) for s in stack_strings]
 
-    return solution_stacks
+    try:
+        return sorted(solution_stacks)
+    except Exception:
+        return solution_stacks
 
 
 def get_application_versions(app_name, version_labels=None, max_records=None, next_token=None):
@@ -514,7 +504,7 @@ def get_all_environments():
     return envs
 
 
-def get_environment(app_name, env_name, env_id=None, include_deleted=False, deleted_back_to=None, want_solution_stack=False):
+def get_environment(app_name=None, env_name=None, env_id=None, include_deleted=False, deleted_back_to=None, want_solution_stack=False):
     LOG.debug('Inside get_environment api wrapper')
 
     kwargs = {}
@@ -599,31 +589,19 @@ def get_new_events(app_name, env_name, request_id,
     # convert to object
     events = []
     for event in result['Events']:
-        try:
-            version_label = event['VersionLabel']
-        except KeyError:
-            version_label = None
-
-        try:
-            environment_name = event['EnvironmentName']
-        except KeyError:
-            environment_name = None
-
-        try:
-            app_name = event['ApplicationName']
-        except KeyError:
-            app_name = None
-
         events.append(
-            Event(message=event['Message'],
-                  event_date=event['EventDate'],
-                  version_label=version_label,
-                  app_name=app_name,
-                  environment_name=environment_name,
-                  severity=event['Severity'],
-                  platform=platform_arn
+            Event(
+                app_name=event.get('ApplicationName'),
+                environment_name=event.get('EnvironmentName'),
+                event_date=event.get('EventDate'),
+                message=event.get('Message'),
+                platform=platform_arn,
+                request_id=event.get('RequestId'),
+                severity=event.get('Severity'),
+                version_label=event.get('VersionLabel')
             )
         )
+
     return events
 
 
@@ -850,3 +828,27 @@ def rebuild_environment(env_id=None, env_name=None):
 
     request_id = result['ResponseMetadata']['RequestId']
     return request_id
+
+
+def get_environment_arn(env_name):
+    return get_environments([env_name])[0].environment_arn
+
+
+def list_tags_for_resource(env_name):
+    response = _make_api_call(
+        'list_tags_for_resource',
+        ResourceArn=get_environment_arn(env_name)
+    )
+
+    return sorted(response['ResourceTags'], key=lambda tag: tag['Key'])
+
+
+def update_tags_for_resource(env_name, tags_to_add, tags_to_remove):
+    response = _make_api_call(
+        'update_tags_for_resource',
+        ResourceArn=get_environment_arn(env_name),
+        TagsToAdd=tags_to_add,
+        TagsToRemove=tags_to_remove
+    )
+
+    return response['ResponseMetadata']['RequestId']
