@@ -1,4 +1,4 @@
-# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -10,17 +10,17 @@
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
-
-from ..core.fileoperations import program_is_installed
+from ebcli.containers import dockerrun
+from ebcli.core.fileoperations import program_is_installed
 import glob
 import os
 
 
 def find_language_type():
-    # Docker could be any language, so we need to check for docker first
+    if smells_of_multi_container_docker():
+        return 'Multi-container Docker'
     if smells_of_docker():
         return 'Docker'
-
     if smells_of_python():
         return 'Python'
     if smells_of_ruby():
@@ -34,14 +34,19 @@ def find_language_type():
     if smells_of_tomcat():
         return 'Tomcat'
 
-    # We cant smell its type
-    ## If there is just an index.html, php will work
+    # If there is just an index.html, PHP will work
     if has_index_html():
         return 'PHP'
 
     return None
 
 
+def smells_of_multi_container_docker():
+    dockerrun_file = dockerrun.get_dockerrun('Dockerrun.aws.json')
+    if dockerrun_file and dockerrun_file.get('AWSEBDockerrunVersion') in (2, '2'):
+        return True
+
+
 def smells_of_docker():
     """
     True if the current directory has a docker file
@@ -128,14 +133,6 @@ def has_platform_definition_file():
     return _contains_file_types('platform.yaml')
 
 
-def is_docker_installed():
-    return program_is_installed('docker')
-
-
-def is_docker_compose_installed():
-    return program_is_installed('docker-compose')
-
-
 def is_boot2docker_installed():
     return program_is_installed('boot2docker')
 
