@@ -1,4 +1,4 @@
-# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -41,6 +41,10 @@ class CommandError(EBCLIException):
         self.code = code
 
 
+class GitRemoteNotSetupError(EBCLIException):
+    pass
+
+
 class ServiceError(EBCLIException):
     """  Error occurred calling the api    """
     def __init__(self, message=None, code=None):
@@ -188,3 +192,7 @@ class UploadError(EBCLIException):
 
 class WorkerQueueNotFound(EBCLIException):
     """ A worker queue could not be found for a worker environment """
+
+
+class DockerVersionError(Exception):
+    pass
