@@ -1,4 +1,4 @@
-# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -18,30 +18,27 @@ from datetime import datetime, timedelta
 import platform
 
 from ..core.fileoperations import _marker
-from semantic_version import Version
 
 from cement.utils.misc import minimal_logger
 from cement.utils.shell import exec_cmd
 from botocore.compat import six
 
 from ebcli.operations import buildspecops
-from ebcli.core.ebglobals import Constants
 from ..core import fileoperations, io
-from ..containers import dockerrun
-from ..lib import aws, ec2, elasticbeanstalk, heuristics, s3, utils, codecommit
-from ebcli.objects.platform import PlatformVersion
+from ..lib import aws, ec2, elasticbeanstalk, heuristics, iam, s3, utils, codecommit
 from ..lib.aws import InvalidParameterValueError
 from ..objects.exceptions import *
-from ..objects.solutionstack import SolutionStack
 from ..objects.sourcecontrol import SourceControl, NoSC
 from ..resources.strings import strings, responses, prompts
+from ..resources.statics import iam_documents, iam_attributes
 
 LOG = minimal_logger(__name__)
 
+
 def wait_for_success_events(request_id, timeout_in_minutes=None,
                             sleep_time=5, stream_events=True, can_abort=False,
                             streamer=None, app_name=None, env_name=None, version_label=None,
-                            platform_arn=None):
+                            platform_arn=None, on_timeout_raise=False, timeout_error_message=None):
     if timeout_in_minutes == 0:
         return
     if timeout_in_minutes is None:
@@ -72,7 +69,12 @@ def wait_for_success_events(request_id, timeout_in_minutes=None,
         if request_id:
             while not events:
                 events = elasticbeanstalk.get_new_events(
-                    None, None, request_id, last_event_time=None, version_label=version_label
+                    app_name,
+                    env_name,
+                    request_id,
+                    last_event_time=None,
+                    platform_arn=platform_arn,
+                    version_label=version_label
                 )
 
                 if len(events) > 0:
@@ -89,14 +91,27 @@ def wait_for_success_events(request_id, timeout_in_minutes=None,
                 else:
                     time.sleep(sleep_time)
 
-        # Get remaining events without request id
+        # Get remaining events
         while (datetime.now() - start) < timediff:
             time.sleep(sleep_time)
 
             events = elasticbeanstalk.get_new_events(
-                app_name, env_name, None, last_event_time=last_time, platform_arn=platform_arn
+                app_name,
+                env_name,
+                request_id,
+                last_event_time=last_time,
+                platform_arn=platform_arn,
+                version_label=version_label
             )
 
+            if events:
+                events = filter_events(
+                    events,
+                    env_name=env_name,
+                    request_id=request_id,
+                    version_label=version_label
+                )
+
             for event in reversed(events):
                 if stream_events:
                     streamer.stream_event(get_event_string(event), safe_to_quit=safe_to_quit)
@@ -110,7 +125,40 @@ def wait_for_success_events(request_id, timeout_in_minutes=None,
     finally:
         streamer.end_stream()
     # We have timed out
-    raise TimeoutError('Timed out while waiting for command to Complete. The timeout can be set using the --timeout option.')
+
+    if not timeout_error_message:
+        timeout_error_message = strings['timeout.error'].format(timeout_in_minutes=timeout_in_minutes)
+
+    if on_timeout_raise:
+        raise TimeoutError(timeout_error_message)
+    else:
+        io.log_error(timeout_error_message)
+
+
+def filter_events(events, version_label=None, request_id=None, env_name=None):
+    """
+    Method filters events by their version_label, request_id, or env_name if supplied,
+    or any combination if multiple are specified.
+
+    :param events: A list of `events` returned by the `DescribeEvents` API
+    :param version_label: An optional `version_label` of the environment to filter by
+    :param request_id: An optional `request_id` of the operation being waited on to filter by
+    :param env_name: An optional `environment_name` of the environment that is being waited on
+    :return: A new list of events filtered as per the rules above.
+    """
+    filtered_events = []
+
+    for event in events:
+        if version_label and event.version_label and (event.version_label != version_label):
+            continue
+        if request_id and event.request_id and (event.request_id != request_id):
+            continue
+        if env_name and event.environment_name and (event.environment_name != env_name):
+            continue
+
+        filtered_events.append(event)
+
+    return filtered_events
 
 
 def wait_for_multiple_success_events(request_ids, timeout_in_minutes=None,
@@ -261,18 +309,23 @@ def wait_for_compose_events(request_id, app_name, grouped_envs, timeout_in_minut
                         successes[index] = True
     finally:
         streamer.end_stream()
-    raise TimeoutError('Timed out while waiting for commands to Complete')
+
+    io.log_error(strings['timeout.error'])
 
 
 def _is_success_string(message):
     if message.startswith(responses['event.completewitherrors']):
         return True
+    if message.startswith(responses['event.launched_environment']):
+        return True
     if message.startswith(responses['event.platformdeletesuccess']):
         return True
     if message.startswith(responses['event.platformdeletefailed']):
         return True
     if message.startswith(responses['event.platformcreatefailed']):
         return True
+    if message.startswith(responses['event.platform_ami_region_service_region_mismatch']):
+        return True
     if message.startswith(responses['event.platformcreatesuccess']):
         return True
     if message == responses['event.greenmessage']:
@@ -317,6 +370,10 @@ def _is_success_string(message):
         raise ServiceError(message)
     if message.startswith(responses['appversion.finished']) and message.endswith('PROCESSED.'):
         return True
+    if responses['tags.tag_update_successful'] in message:
+        return True
+    if responses['tags.no_tags_to_update'] in message:
+        return True
 
     return False
 
@@ -400,208 +457,6 @@ def get_app_version_s3_location(app_name, version_label):
     return s3_bucket, s3_key
 
 
-def list_platform_versions_sorted_by_name(platform_name=None, status=None, owner=None, show_status=False, excludeEBOwned=True):
-    platform_list = elasticbeanstalk.list_platform_versions(
-        platform_name=platform_name, status=status, owner=owner)
-    platform_tuples = list()
-    platforms = list()
-    arn_index = 0
-    status_index = 1
-
-    for platform in platform_list:
-        arn = platform['PlatformArn']
-        owner = platform['PlatformOwner']
-
-        if excludeEBOwned and owner == Constants.AWS_ELASTIC_BEANSTALK_ACCOUNT:
-            continue
-
-        _, platform_name, _ = PlatformVersion.arn_to_platform(arn)
-
-        platform_tuples.append((arn, platform['PlatformStatus']))
-
-    # Sort by name, then by version
-    platform_tuples.sort(key=lambda platform: (PlatformVersion.get_platform_name(platform[arn_index]), Version(PlatformVersion.get_platform_version(platform[arn_index]))), reverse=True)
-
-    for platform in platform_tuples:
-        if show_status:
-            platforms.append("%s  Status: %s" % (platform[arn_index], platform[status_index]))
-        else:
-            platforms.append(platform[arn_index])
-
-    return platforms
-
-
-def get_custom_platform(custom_platforms):
-    accounts = dict()
-    platform_names = []
-
-    for platform_arn in custom_platforms:
-        account_id, platform_name, platform_version = PlatformVersion.arn_to_platform(platform_arn)
-
-        try:
-            platforms = accounts[account_id]
-        except KeyError:
-            platforms = dict()
-
-        try:
-            platform = platforms[platform_name]
-        except KeyError:
-            platform = dict()
-
-        platform[platform_version] = platform_arn
-        platforms[platform_name] = platform
-        accounts[account_id] = platforms
-        platform_names.append(platform_name)
-
-    longest_platform_name = len(max(platform_names, key=len))
-
-    platforms = []
-    for account in accounts:
-        for platform in accounts[account]:
-            platforms.append("%-*s (Owned by: %s)" % (longest_platform_name, platform, account))
-
-    io.echo()
-    io.echo(prompts['platform.prompt'])
-    selected_platform = utils.prompt_for_item_in_list(platforms)
-
-    option_pattern = re.compile('^(.+)\s+\(Owned by: (.+)\)$')
-    match = option_pattern.match(selected_platform)
-
-    selected_platform_name, selected_account = match.group(1, 2)
-    selected_platform_name = selected_platform_name.strip()
-
-    versions = []
-    for version in accounts[selected_account][selected_platform_name]:
-        versions.append(version)
-
-    if len(versions) > 1:
-        versions.sort(key=lambda arn: Version(arn), reverse=True)
-        io.echo()
-        io.echo(prompts['sstack.version'])
-        version = utils.prompt_for_item_in_list(versions)
-    else:
-        version = versions[0]
-
-    return PlatformVersion(accounts[selected_account][selected_platform_name][version])
-
-
-def prompt_for_solution_stack(module_name=None):
-    solution_stacks = elasticbeanstalk.get_available_solution_stacks()
-    custom_platforms = list_platform_versions_sorted_by_name()
-    custom_platform_option = "Custom Platform"
-
-    # get list of platforms
-    platforms = []
-    for stack in solution_stacks:
-        if stack.platform not in platforms:
-            platforms.append(stack.platform)
-
-    if custom_platforms is not None and len(custom_platforms) > 0:
-        platforms.append(custom_platform_option)
-
-    cwd = os.getcwd()
-    # First check to see if we know what language the project is in
-    try:
-        fileoperations._traverse_to_project_root()
-        platform = heuristics.find_language_type()
-
-        if platform == 'Docker':
-            # Check to see if dockerrun is version one or two
-            dockerrun_file = dockerrun.get_dockerrun(
-                os.path.join(os.getcwd(), 'Dockerrun.aws.json'))
-            if dockerrun_file:
-                if dockerrun_file.get('AWSEBDockerrunVersion') in (1, '1'):
-                    platform = 'Docker'
-                else:
-                    platform = 'Multi-container Docker'
-    finally:
-        os.chdir(cwd)
-
-    if platform is not None:
-        io.echo()
-        io.echo(prompts['platform.validate'].replace('{platform}', platform))
-        correct = io.get_boolean_response()
-
-    if not platform or not correct:
-        # ask for platform
-        io.echo()
-        io.echo(prompts['platform.prompt']
-                if not module_name
-                else prompts['platform.prompt.withmodule'].replace('{module_name}',
-                                                                   module_name))
-        platform = utils.prompt_for_item_in_list(platforms)
-
-    if platform == custom_platform_option:
-        return get_custom_platform(custom_platforms)
-
-    # filter
-    solution_stacks = [x for x in solution_stacks if x.platform == platform]
-
-    #get Versions
-    versions = []
-    for stack in solution_stacks:
-        if stack.version not in versions:
-            versions.append(stack.version)
-
-    #now choose a version (if applicable)
-    if len(versions) > 1:
-        io.echo()
-        io.echo(prompts['sstack.version'])
-        version = utils.prompt_for_item_in_list(versions)
-    else:
-        version = versions[0]
-
-    return get_latest_solution_stack(version, stack_list=solution_stacks)
-
-
-def get_latest_platform(platform_arn):
-    account_id, platform_name, platform_version = PlatformVersion.arn_to_platform(platform_arn)
-
-    platforms = elasticbeanstalk.list_platform_versions(platform_name, 'latest', 'Ready', account_id)
-
-    if len(platforms) == 1:
-        return PlatformVersion(platforms[0]['PlatformArn'])
-
-    # The customer doesn't have access to the platform anymore?
-    return PlatformVersion(platform_arn)
-
-
-def get_latest_solution_stack(platform_version, stack_list=None):
-    if PlatformVersion.is_valid_arn(platform_version):
-        return get_latest_platform(platform_version)
-
-    if stack_list:
-        solution_stacks = stack_list
-    else:
-        solution_stacks = elasticbeanstalk.\
-            get_available_solution_stacks()
-
-    #filter
-    solution_stacks = [x for x in solution_stacks
-                       if x.version == platform_version]
-
-    #Lastly choose a server type
-    servers = []
-    for stack in solution_stacks:
-        if stack.server not in servers:
-            servers.append(stack.server)
-
-    # Default to latest version of server
-    # We are assuming latest is always first in list.
-    if len(servers) < 1:
-        raise NotFoundError(strings['sstacks.notaversion'].
-                            replace('{version}', platform_version))
-    server = servers[0]
-
-    #filter
-    solution_stacks = [x for x in solution_stacks if x.server == server]
-
-    #should have 1 and only have 1 result
-    assert len(solution_stacks) == 1, 'Filtered Solution Stack list ' \
-                                      'contains multiple results'
-    return solution_stacks[0]
-
-
 def create_app(app_name, default_env=None):
     # Attempt to create app
     try:
@@ -658,7 +513,7 @@ def pull_down_app_info(app_name, default_env=None):
     if keyname is None:
         keyname = -1
 
-    return env.platform.version, keyname
+    return env.platform.platform_shorthand, keyname
 
 
 def open_webpage_in_browser(url, ssl=False):
@@ -820,7 +675,7 @@ def create_app_version(app_name, process=False, label=None, message=None, staged
     fileoperations._traverse_to_project_root()
     try:
         if heuristics.directory_is_empty():
-            io.log_warning(strings['appversion.none'])
+            io.echo('NOTE: {}'.format(strings['appversion.none']))
             return None
     finally:
         os.chdir(cwd)
@@ -951,7 +806,7 @@ def create_app_version_from_source(app_name, source, process=False, label=None,
     fileoperations._traverse_to_project_root()
     try:
         if heuristics.directory_is_empty():
-            io.log_warning(strings['appversion.none'])
+            io.echo('NOTE: {}'.format(strings['appversion.none']))
             return None
     finally:
         os.chdir(cwd)
@@ -1075,11 +930,10 @@ def update_environment(env_name, changes, nohang, remove=None,
         return
 
     io.echo('Printing Status:')
-    try:
-        wait_for_success_events(request_id, timeout_in_minutes=timeout,
-                                can_abort=True)
-    except TimeoutError:
-        io.log_error(strings['timeout.error'])
+
+    wait_for_success_events(request_id, timeout_in_minutes=timeout,
+                            can_abort=True)
+
 
 # BRANCH-DEFAULTS FOR CONFIG FILE
 def write_setting_to_current_branch(keyname, value):
@@ -1131,10 +985,6 @@ def get_default_region():
         return None
 
 
-def get_default_solution_stack():
-    return get_config_setting_from_branch_or_default('default_platform')
-
-
 def get_setting_from_current_branch(keyname):
     try:
         source_control = SourceControl.get_source_control()
@@ -1163,58 +1013,6 @@ def get_config_setting_from_branch_or_default(key_name, default=_marker):
         return fileoperations.get_config_setting('global', key_name, default=default)
 
 
-def is_platform_arn(solution_string):
-    if solution_string is None:
-        return False
-
-    return PlatformVersion.ARN_PATTERN.match(str(solution_string)) is not None
-
-
-def get_solution_stack(solution_string):
-    #If string is explicit, do not check
-    if re.match(r'^\d\dbit [\w\s]+[0-9.]* v[0-9.]+ running .*$',
-                solution_string):
-        return SolutionStack(solution_string)
-
-    solution_string = solution_string.lower()
-    solution_stacks = elasticbeanstalk.get_available_solution_stacks()
-
-    # check for exact string
-    stacks = [x for x in solution_stacks if x.name.lower() == solution_string]
-
-    if len(stacks) == 1:
-        return stacks[0]
-
-    #should only have 1 result
-    if len(stacks) > 1:
-        LOG.error('Platform list contains '
-                  'multiple results')
-        return None
-
-    # No exact match, check for versions
-    string = solution_string.replace('-', ' ')
-    # put dash back in preconfigured types
-    string = re.sub('preconfigured\\s+docker', 'preconfigured - docker', string)
-    # put dash back in multi-container types
-    string = re.sub('multi\\s+container', 'multi-container', string)
-    string = re.sub(r'([a-z])([0-9])', '\\1 \\2', string)
-    stacks = [x for x in solution_stacks if x.version.lower() == string]
-
-    if len(stacks) > 0:
-        # Give the latest version. Latest is always first in list
-        return stacks[0]
-
-    # No exact match, check for platforms
-    stacks = [x for x in solution_stacks if x.platform.lower() == string]
-
-    if len(stacks) > 0:
-        # Give the latest version. Latest is always first in list
-        return stacks[0]
-
-    raise NotFoundError(prompts['sstack.invalidkey'].replace('{string}',
-                                                             solution_string))
-
-
 def is_cname_available(cname):
     return elasticbeanstalk.is_cname_available(cname)
 
@@ -1301,3 +1099,42 @@ def wait_for_processed_app_versions(app_name, version_labels, timeout=5):
         io.log_error(strings['appversion.cannotdeploy'])
         return False
     return True
+
+
+def create_default_instance_profile(profile_name=iam_attributes.DEFAULT_ROLE_NAME):
+    """ Create default elasticbeanstalk IAM profile and return its name. """
+    create_instance_profile(profile_name, iam_attributes.DEFAULT_ROLE_POLICIES)
+    return profile_name
+
+
+def create_instance_profile(profile_name, policy_arns, role_name=None, inline_policy_name=None, inline_policy_doc=None):
+    """ Create instance profile and associated IAM role, and attach policy ARNs. 
+        If role_name is omitted profile_name will be used as role name.
+        Inline policy is optional. """
+    try:
+        name = iam.create_instance_profile(profile_name)
+        if name:
+            io.log_info('Created instance profile: {}.'.format(name))
+
+        if not role_name:
+            role_name = profile_name
+        name = _create_instance_role(role_name, policy_arns)
+        if name:
+            io.log_info('Created instance role: {}.'.format(name))
+
+        if inline_policy_name:
+            iam.put_role_policy(role_name, inline_policy_name, inline_policy_doc)
+
+        iam.add_role_to_profile(profile_name, role_name)
+
+    except NotAuthorizedError:
+        # Not a root account. Just assume role exists
+        io.log_warning(strings['platformcreateiamdescribeerror.info'].format(profile_name=profile_name))
+
+    return profile_name
+
+
+def _create_instance_role(role_name, policy_arns):
+    document = iam_documents.EC2_ASSUME_ROLE_PERMISSION
+    ret = iam.create_role_with_policy(role_name, document, policy_arns)
+    return ret
