@@ -1,4 +1,4 @@
-# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -18,7 +18,7 @@ from . import commonops
 
 def open_app(app_name, env_name):
     # get cname
-    env = elasticbeanstalk.get_environment(app_name, env_name)
+    env = elasticbeanstalk.get_environment(app_name=app_name, env_name=env_name)
     settings = elasticbeanstalk.describe_configuration_settings(
         app_name, env_name)
 
