@@ -1,4 +1,4 @@
-# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -18,7 +18,7 @@ from . import commonops, gitops
 
 def switch_default_environment(app_name, env_name):
     # check that environment exists
-    elasticbeanstalk.get_environment(app_name, env_name)
+    elasticbeanstalk.get_environment(app_name=app_name, env_name=env_name)
     commonops.set_environment_for_current_branch(env_name)
 
 
