@@ -1,4 +1,4 @@
-# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -43,6 +43,10 @@ strings = {
     'create.info': 'Creates a new environment.',
     'create.epilog': 'Type "--vpc." or "--database." for more VPC and database options.',
     'create.missinggroup': 'A group name is required when creating multiple environments. Please use the --group option.',
+    'create.sample_application_download_option': 'Do you want to download the sample application into the current directory?',
+    'create.downloading_sample_application': 'Downloading sample application to the current directory.',
+    'create.sample_application_download_complete': 'Download complete.',
+    'create.download_sample_app_choice_error': "'{choice}' is not a valid choice.",
     'events.info': 'Gets recent events.',
     'open.info': 'Opens the application URL in a browser.',
     'console.info': 'Opens the environment in the AWS Elastic Beanstalk Management Console.',
@@ -75,14 +79,7 @@ strings = {
                              'patch-increment based on the most recent platform version. The version and increment '
                              'options are mutually exclusive.',
 
-    'platformcreateiamdescribeerror.info': "Insufficient IAM privileges. Unable to determine if instance profile '{0}'exists.".format(iam_attributes.DEFAULT_PLATFORM_BUILDER_ROLE),
-    'platformcreateiamcreated.info': "Created instance profile '{0}'".format(iam_attributes.DEFAULT_PLATFORM_BUILDER_ROLE),
-    'platformcreateiamcreateerror.info': "Insufficient IAM privileges. Unable to create instance profile '{0}', assuming that it exists.".format(
-                    iam_attributes.DEFAULT_PLATFORM_BUILDER_ROLE),
-
-    'platformcreateiampolicyadded.info': "Added required permissions to instance profile '{0}'".format(
-        iam_attributes.DEFAULT_PLATFORM_BUILDER_ROLE),
-
+    'platformcreateiamdescribeerror.info': "Insufficient IAM privileges. Unable to determine if instance profile '{profile_name}' exists, assuming that it exists.",
 
     'platformdelete.info': 'Deletes platform resources.',
     'platformdeleteversion.info': 'Deletes a custom platform version.',
@@ -145,6 +142,7 @@ strings = {
     # Error when --sample and --label flag are both used on create
     'create.sampleandlabel': 'You cannot use the "--sample" and "--version" options together.',
     'create.singleandsize': 'You cannot use the "--single" and "--scale" options together.',
+    'create.single_and_elb_type': 'You cannot use the "--single" and "--elb-type" options together.',
     'create.appdoesntexist': 'The specified app {app_name} does not exist. Skipping.',
     'create.missinggroupsuffix': 'The environment name specified in env.yaml ends with a \'+\', but no group suffix was provided. Please pass the --env-group-suffix argument.',
     'ssh.instanceandnumber': 'You cannot use the "--instance" and "--number" options together.',
@@ -167,7 +165,7 @@ strings = {
     # Error, no solution stacks returned. Almost always due to permissions
     'sstacks.notfound': 'Elastic Beanstalk could not find any platforms. Ensure you have the necessary permissions to access Elastic Beanstalk.',
     'sstacks.notaversion': 'Elastic Beanstalk could not find any supported platforms for the given version {version}.',
-    'timeout.error': 'The operation timed out. The state of the environment is unknown. The timeout can be set using the --timeout option.',
+    'timeout.error': "The EB CLI timed out after {timeout_in_minutes} minute(s). The operation might still be running. To keep viewing events, run 'eb events -f'. To set timeout duration, use '--timeout MINUTES'.",
     'sc.notfound': 'Git is not set up for this project. EB CLI will deploy a .zip file of the entire directory.',
     'exit.platformworkspacenotsupported': 'This command is not supported for Platform workspaces.',
     'exit.applicationworkspacenotsupported': 'This command is not supported for Application workspaces.',
@@ -195,7 +193,7 @@ strings = {
                             'Variables must start with a letter.',
     'tags.invalidformat': 'You must provide a comma-separated list using the format name=value to set tags. '
                           'Tags may only contain letters, numbers, and the following symbols: / _ . : + = - @',
-    'tags.max': 'Elastic Beanstalk supports a maximum of 7 tags.',
+    'tags.max': 'Elastic Beanstalk supports a maximum of 50 tags.',
     'deploy.invalidoptions': 'You cannot use the "--version" option with either the "--message" or "--label" option.',
     'init.getvarsfromoldeb': 'You previous used an earlier version of eb. Getting options from .elasticbeanstalk/config.\n'
                              'Credentials will now be stored in ~/.aws/config',
@@ -337,6 +335,24 @@ strings = {
     'platformstatus.upgrade': 'A more recent version of this platform is available. Type \'eb upgrade\' to uprade the platform version used by this environment.',
     'platform.nobuilderenv': 'This workspace has not yet been associated with a builder environment. One will be configured once you create a platform version.',
     'codebuild.buildlogs': 'You can find logs for the CodeBuild build here: {logs_link}',
+
+    # Tags
+    'tags.duplicate_across_delete_and_update_lists': "A tag with the key '{0}' is specified for both '--delete' and '--update'. You can either delete or update each tag in a single operation.",
+    'tags.duplicate_key_in_add_list': "A tag with the key '{0}' is specified more than once for '--add'. You can add a tag key only once.",
+    'tags.duplicate_key_in_delete_list': "A tag with the key '{0}' is specified more than once for '--delete'. You can delete a tag key only once.",
+    'tags.duplicate_key_in_update_list': "A tag with the key '{0}' is specified more than once for '--update'. You can update a tag key only once.",
+    'tags.invalid_tag_key': "Tag key '{0}' has invalid characters. Only letters, numbers, white space, and these characters are allowed: _ . : / + - @.",
+    'tags.invalid_tag_value': "Tag value '{0}' has invalid characters. Only letters, numbers, white space, and these characters are allowed: _ . : / = + - @.",
+    'tags.list_with_other_arguments': "You can't specify the '--list' option with the '--add', '--delete', or '--update' option.",
+    'tags.resource_tags_missing': "The response of the 'list_tags_for_resource' API call is missing the 'ResourceTags' field.",
+    'tags.tag_keys_already_exist': "Tags with the following keys can't be added because they already exist:",
+    'tags.tag_key_cant_be_blank': 'Tag key must not be blank.',
+    'tags.tag_value_cant_be_blank': 'Tag value must not be blank.',
+    'tags.tag_keys_dont_exist_for_deletion': "Tags with the following keys can't be deleted because they don't exist:",
+    'tags.tag_keys_dont_exist_for_update': "Tags with the following keys can't be updated because they don't exist:",
+    'tags.tag_key_max_length_exceeded': "Tag with the following key exceed length limit. Tag keys can be up to 128 characters in length.",
+    'tags.tag_value_max_length_exceeded': "Tag with the following value exceed length limit. Tag values can be up to 256 characters in length.",
+    'cloudformation.cannot_find_app_source_for_environment': 'Cannot find app source for environment'
 }
 prompts = {
     'events.hanging': 'Streaming new events. Use CTRL+C to exit.',
@@ -390,7 +406,7 @@ prompts = {
     'vpc.ec2subnets': 'Enter a comma-separated list of Amazon EC2 subnets',
     'vpc.elbsubnets': 'Enter a comma-separated list of Amazon ELB subnets',
     'vpc.securitygroups': 'Enter a comma-separated list of Amazon VPC security groups',
-    'vpc.elbpublic': 'Do you want the load balencer to be public? (Select no for internal)',
+    'vpc.elbpublic': 'Do you want the load balancer to be public? (Select no for internal)',
     'vpc.dbsubnets': 'Enter a comma-separated list of database subnets',
     'logs.retrieving': 'Retrieving logs...',
     'swap.envprompt': 'Select the environment with which you want to swap CNAMEs.',
@@ -408,7 +424,7 @@ prompts = {
     'upgrade.applyrolling': 'Enabling {0}-based rolling updates to environment.',
     'create.dockerrunupgrade': 'Multicontainer Docker environments do not support the version number of the Dockerrun.aws.json file that you provided. Type "eb labs convert-dockerrun" to convert it to a newer format.',
     'ecs.permissions': 'The Multi-container Docker platform requires additional ECS permissions. Add the permissions to the aws-elasticbeanstalk-ec2-role or use your own instance profile by typing "-ip {profile-name}".\n'
-                       'For more information see: https://docs.aws.amazon.com/elasticbeanstalk/latest/dg/create_deploy_docker_ecstutorial.html#create_deploy_docker_ecstutorial_role',
+                       'For more information see: https://docs.aws.amazon.com/elasticbeanstalk/latest/dg/create_deploy_docker_ecs.html#create_deploy_docker_ecs_role',
     'create.servicerole.info': '2.0+ Platforms require a service role. We will attempt to create one for you. You can specify your own role using the --service-role option.',
     'create.servicerole.view': 'Type "view" to see the policy, or just press ENTER to continue',
     'create.servicerole.required': '2.0+ Platforms require a service role. You can provide one with --service-role option',
@@ -629,13 +645,23 @@ flag_text = {
 
     # lifecycle
     'lifecycle.print': 'prints the current application version lifecycle policy',
-    'lifecycle.update': 'allows an inline update to a application version lifecycle policy'
+    'lifecycle.update': 'allows an inline update to a application version lifecycle policy',
+
+    # Tags
+    'tags.add': 'create new environment tags provided as a comma-separated list of key=value pairs',
+    'tags.delete': 'delete existing environment tags provided as a comma-separated list of keys',
+    'tags.env': 'environment on which to perform tags operation',
+    'tags.info': 'Allows adding, deleting, updating, and listing of environment tags.',
+    'tags.list': 'lists all environment resource tags',
+    'tags.update': 'update existing environment tags provided as a comma-separated list of keys=value pairs',
 }
 
 
 ### The below are programmatic and are not intended to be edited unless the service response changes
 responses = {
     'event.completewitherrors': 'Create environment operation is complete, but with errors.',
+    'event.launched_environment': 'Launched environment',
+    'event.platform_ami_region_service_region_mismatch': 'Unmatched region for created AMI',
     'event.platformdeletesuccess': 'Successfully deleted platform version',
     'event.platformdeletefailed': 'Failed to delete platform version',
     'event.platformcreatefailed': 'Failed to create platform version',
@@ -678,9 +704,11 @@ responses = {
     'create.noplatform': 'Unable to determine base for template pack (no solution stack)',
     'create.ecsdockerrun1': 'ECS Application sourcebundle validation error: Unsupported AWSEBDockerrunVersion:',
     'appversion.finished': 'Finished processing application version',
+    'tags.tag_update_successful': 'Environment tag update completed successfully.',
+    'tags.no_tags_to_update': 'Environment tag update failed.',
 
     # Env Restore display table
-    'restore.norestore': 'Environment will not be restored'
+    'restore.norestore': 'Environment will not be restored',
 }
 git_ignore = [
     '# Elastic Beanstalk Files',        # comment line
